use serde::Deserialize;
use serde::Serialize;

use super::ast_node::WasmAstNode;
use super::macros::to_wasm;
use super::to_wasm::ToWasm;
use super::AdditionalProperties;
use crate::compare::ElispFact;
use crate::types::DiarySexp;
use crate::types::GetAffiliatedKeywords;
use crate::wasm::to_wasm::ToWasmStandardProperties;

#[derive(Debug, Serialize, Deserialize)]
pub struct WasmDiarySexp {
    #[serde(flatten)]
    pub(crate) additional_properties: AdditionalProperties,
    pub(crate) value: String,
}

to_wasm!(
    WasmDiarySexp,
    DiarySexp<'s>,
    original,
    wasm_context,
    { WasmAstNode::DiarySexp(original) },
    { "diary-sexp".into() },
    {
        let additional_properties = original
            .get_affiliated_keywords()
            .to_wasm(wasm_context.clone())?;

        Ok((
            Vec::new(),
            WasmDiarySexp {
                additional_properties,
                value: original.value.to_owned(),
            },
        ))
    }
);
