use serde::Deserialize;
use serde::Serialize;

use super::ast_node::WasmAstNode;
use super::macros::to_wasm;
use super::to_wasm::ToWasm;
use crate::compare::ElispFact;
use crate::types::ExportSnippet;
use crate::wasm::to_wasm::ToWasmStandardProperties;

#[derive(Debug, Serialize, Deserialize)]
pub struct WasmExportSnippet {
    #[serde(rename = "back-end")]
    pub(crate) backend: String,
    pub(crate) value: Option<String>,
}

to_wasm!(
    WasmExportSnippet,
    ExportSnippet<'s>,
    original,
    wasm_context,
    { WasmAstNode::ExportSnippet(original) },
    { "export-snippet".into() },
    {
        Ok((
            Vec::new(),
            WasmExportSnippet {
                backend: original.backend.to_owned(),
                value: original.contents.map(|s| s.to_owned()),
            },
        ))
    }
);
