use serde::Deserialize;
use serde::Serialize;

use super::ast_node::WasmAstNode;
use super::macros::to_wasm;
use super::to_wasm::ToWasm;
use crate::compare::ElispFact;
use crate::types::TableRow;
use crate::types::TableRowType;
use crate::wasm::to_wasm::ToWasmStandardProperties;

#[derive(Debug, Serialize, Deserialize)]
pub struct WasmTableRow {
    #[serde(rename = "type")]
    pub(crate) row_type: String,
}

to_wasm!(
    WasmTableRow,
    TableRow<'s>,
    original,
    wasm_context,
    { WasmAstNode::TableRow(original) },
    { "table-row".into() },
    {
        let children = original
            .children
            .iter()
            .map(|child| {
                child
                    .to_wasm(wasm_context.clone())
                    .map(Into::<WasmAstNode>::into)
            })
            .collect::<Result<Vec<_>, _>>()?;

        Ok((
            children,
            WasmTableRow {
                row_type: match original.get_type() {
                    TableRowType::Standard => "standard",
                    TableRowType::Rule => "rule",
                }
                .to_owned(),
            },
        ))
    }
);
